import React, { Fragment, createContext, useContext, useMemo, useEffect, useCallback, useState } from 'react';
import { pdfjs, Document, Page } from 'react-pdf';

var FileType;
(function (FileType) {
  FileType["PDF"] = "pdf";
})(FileType || (FileType = {}));
var BlockType;
(function (BlockType) {
  BlockType["PAGE"] = "PAGE";
  BlockType["LINE"] = "LINE";
  BlockType["WORD"] = "WORD";
  BlockType["TABLE"] = "TABLE";
  BlockType["CELL"] = "CELL";
  BlockType["SELECTION_ELEMENT"] = "SELECTION_ELEMENT";
})(BlockType || (BlockType = {}));

var DefaultPageRender = function DefaultPageRender(props) {
  return React.createElement(Fragment, null, props.children);
};

function _extends() {
  _extends = Object.assign ? Object.assign.bind() : function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];
      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }
    return target;
  };
  return _extends.apply(this, arguments);
}
function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;
  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }
  return target;
}

var styles = {"defaultRenderText":"_3Q0a-","selected":"_3Qv-g"};

var _excluded = ["width", "height", "selected"];
var DefaultTextRenderer = function DefaultTextRenderer(_ref) {
  var width = _ref.width,
    height = _ref.height,
    selected = _ref.selected,
    block = _objectWithoutPropertiesLoose(_ref, _excluded);
  var defaultHeight = Math.max(15, block.Geometry.BoundingBox.Height * height);
  return React.createElement("div", {
    key: block.Id,
    className: styles.defaultRenderText + " " + (selected ? styles.selected : ''),
    style: {
      width: block.Geometry.BoundingBox.Width * width + 10 + "px",
      minHeight: defaultHeight + 4 + "px",
      fontSize: "calc(" + defaultHeight + "px * 0.7)"
    }
  }, block.Text);
};

var styles$1 = {"overlayContent":"_1gKhP"};

var ReactOCRRendererContext = createContext(undefined);
var ActionType;
(function (ActionType) {
  ActionType["SET_PAGES"] = "SET_PAGES";
  ActionType["SET_CURRENT_BLOCK"] = "SET_CURRENT_BLOCK";
  ActionType["SET_CURRENT_BLOCK_INDEX"] = "SET_CURRENT_BLOCK_INDEX";
})(ActionType || (ActionType = {}));
var reducer = function reducer(state, action) {
  switch (action.type) {
    case ActionType.SET_PAGES:
      return _extends({}, state, {
        pages: action.payload
      });
    case ActionType.SET_CURRENT_BLOCK:
      return _extends({}, state, {
        currentBlock: action.payload
      });
    case ActionType.SET_CURRENT_BLOCK_INDEX:
      return _extends({}, state, {
        currentBlockIndex: action.payload
      });
    default:
      return state;
  }
};
var useReactOCRRenderer = function useReactOCRRenderer() {
  var _React$useReducer = React.useReducer(reducer, {
      pages: {},
      currentBlock: undefined,
      currentBlockIndex: 0
    }),
    _React$useReducer$ = _React$useReducer[0],
    pages = _React$useReducer$.pages,
    currentBlock = _React$useReducer$.currentBlock,
    currentBlockIndex = _React$useReducer$.currentBlockIndex,
    dispatch = _React$useReducer[1];
  var context = useContext(ReactOCRRendererContext);
  var scrollToBlockId = function scrollToBlockId(blockId) {
    setTimeout(function () {
      var block = document.getElementById(blockId);
      if (block) {
        block.scrollIntoView({
          behavior: 'smooth',
          inline: 'center',
          block: 'center'
        });
      }
    }, 250);
  };
  var _useMemo = useMemo(function () {
      return Object.values(pages).reduce(function (acc, val) {
        val.forEach(function (block) {
          acc[0][block.Id] = block;
          acc[1].push(block.Id);
        });
        return acc;
      }, [{}, []]);
    }, [pages]),
    blocks = _useMemo[0],
    blockIds = _useMemo[1];
  useEffect(function () {
    if (!blockIds.length) return;
    dispatch({
      type: ActionType.SET_CURRENT_BLOCK,
      payload: blocks[blockIds[currentBlockIndex]]
    });
  }, [blockIds]);
  var scrollToBlock = useCallback(function (index) {
    if (!blockIds.length) return;
    dispatch({
      type: ActionType.SET_CURRENT_BLOCK_INDEX,
      payload: index
    });
  }, [blockIds]);
  var nextBlock = useCallback(function () {
    dispatch({
      type: ActionType.SET_CURRENT_BLOCK_INDEX,
      payload: currentBlockIndex === blockIds.length - 1 ? 0 : currentBlockIndex + 1
    });
  }, [blockIds, currentBlockIndex]);
  useEffect(function () {
    dispatch({
      type: ActionType.SET_CURRENT_BLOCK,
      payload: blocks[blockIds[currentBlockIndex]]
    });
  }, [currentBlockIndex]);
  useEffect(function () {
    if (!currentBlock) return;
    scrollToBlockId(currentBlock.Id);
  }, [currentBlock]);
  var prevBlock = useCallback(function () {
    dispatch({
      type: ActionType.SET_CURRENT_BLOCK_INDEX,
      payload: currentBlockIndex === 0 ? blockIds.length - 1 : currentBlockIndex - 1
    });
  }, [blockIds, currentBlockIndex]);
  var setPages = useCallback(function (pages) {
    dispatch({
      type: ActionType.SET_PAGES,
      payload: pages
    });
  }, []);
  var contextPreset = useMemo(function () {
    return {
      pages: pages,
      setPages: setPages,
      scrollToBlock: scrollToBlock,
      nextBlock: nextBlock,
      prevBlock: prevBlock,
      currentBlock: currentBlock,
      blockIds: blockIds,
      scrollToBlockId: scrollToBlockId
    };
  }, [pages, setPages, currentBlock, blockIds, scrollToBlock, nextBlock, prevBlock]);
  return context !== null && context !== void 0 ? context : contextPreset;
};
var ReactOCRRendererContextProvider = function ReactOCRRendererContextProvider(_ref) {
  var children = _ref.children;
  var value = useReactOCRRenderer();
  return React.createElement(ReactOCRRendererContext.Provider, {
    value: value
  }, children);
};

var OCROverlayRenderer = function OCROverlayRenderer(_ref) {
  var blocks = _ref.blocks,
    width = _ref.width,
    height = _ref.height,
    customTextRenderComponent = _ref.customTextRenderComponent;
  var RenderText = customTextRenderComponent !== null && customTextRenderComponent !== void 0 ? customTextRenderComponent : DefaultTextRenderer;
  var _useReactOCRRenderer = useReactOCRRenderer(),
    currentBlock = _useReactOCRRenderer.currentBlock;
  return React.createElement("div", {
    className: styles$1.overlayContent
  }, React.createElement("div", {
    className: styles$1.ocrPage
  }, blocks === null || blocks === void 0 ? void 0 : blocks.map(function (block) {
    return React.createElement("div", {
      key: block.Id,
      id: block.Id,
      style: {
        left: block.Geometry.BoundingBox.Left * width + "px",
        top: block.Geometry.BoundingBox.Top * height + "px",
        position: 'absolute'
      }
    }, React.createElement(RenderText, Object.assign({}, block, {
      selected: (currentBlock === null || currentBlock === void 0 ? void 0 : currentBlock.Id) === block.Id,
      width: width,
      height: height
    })));
  })));
};

var styles$2 = {"pageContainer":"_2S3gd","ocrBlock":"_23n6h","ocrPage":"_3IMu9"};

pdfjs.GlobalWorkerOptions.workerSrc = "//unpkg.com/pdfjs-dist@" + pdfjs.version + "/build/pdf.worker.min.js";
var OCRPdfRenderer = function OCRPdfRenderer(_ref) {
  var file = _ref.file,
    _ref$ocrData = _ref.ocrData,
    ocrData = _ref$ocrData === void 0 ? [] : _ref$ocrData,
    customTextRenderComponent = _ref.customTextRenderComponent,
    customPageRenderComponent = _ref.customPageRenderComponent,
    _ref$size = _ref.size,
    size = _ref$size === void 0 ? {
      width: 500
    } : _ref$size,
    _ref$showOnlyResultPa = _ref.showOnlyResultPages,
    showOnlyResultPages = _ref$showOnlyResultPa === void 0 ? false : _ref$showOnlyResultPa,
    _ref$showOCROverlay = _ref.showOCROverlay,
    showOCROverlay = _ref$showOCROverlay === void 0 ? true : _ref$showOCROverlay;
  var _useState = useState([]),
    pages = _useState[0],
    setPages = _useState[1];
  var _useState2 = useState({
      width: size.width,
      height: 0
    }),
    _useState2$ = _useState2[0],
    width = _useState2$.width,
    height = _useState2$.height,
    setPageSize = _useState2[1];
  var handlePageLoad = useCallback(function () {
    try {
      if (!file) {
        return Promise.resolve();
      }
      return Promise.resolve(pdfjs.getDocument(file).promise).then(function (docData) {
        var pageCount = docData._pdfInfo.numPages;
        var pagePromises = Array.from({
          length: pageCount
        }, function (_, pageNumber) {
          return docData.getPage(pageNumber + 1).then(function (pageData) {
            return pageData.getTextContent().then(function (textContent) {
              return textContent.items.map(function (v) {
                return v.str;
              }).join(' ');
            });
          });
        });
        return Promise.all(pagePromises).then(function (p) {
          setPages(p);
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }, [file]);
  useEffect(function () {
    handlePageLoad();
  }, [file, handlePageLoad]);
  var RenderPage = customPageRenderComponent !== null && customPageRenderComponent !== void 0 ? customPageRenderComponent : DefaultPageRender;
  var rootList = Object.keys(showOnlyResultPages ? ocrData : pages);
  return React.createElement(Document, {
    file: file,
    onLoadSuccess: function onLoadSuccess(p) {
      p.getPage(1).then(function (page) {
        setPageSize({
          width: width || size.width,
          height: width * (page.view[3] / page.view[2])
        });
      });
    }
  }, rootList.map(function (pString) {
    var _ocrData$i, _ocrData$i2;
    var i = parseInt(pString, 10);
    return React.createElement("div", {
      key: "react-ocr-renderer-pdf-page-" + i,
      className: styles$2.pageContainer,
      style: {
        width: width
      }
    }, React.createElement(RenderPage, {
      page: i,
      blocks: (_ocrData$i = ocrData[i]) !== null && _ocrData$i !== void 0 ? _ocrData$i : []
    }, React.createElement(Page, {
      renderTextLayer: false,
      width: width,
      pageNumber: i,
      renderAnnotationLayer: false
    }), showOCROverlay && React.createElement(OCROverlayRenderer, {
      blocks: (_ocrData$i2 = ocrData[i]) !== null && _ocrData$i2 !== void 0 ? _ocrData$i2 : [],
      width: width,
      height: height,
      customTextRenderComponent: customTextRenderComponent
    })));
  }));
};

var groupPages = function groupPages(blocks) {
  var pages = {};
  blocks.forEach(function (block) {
    pages[block.Page] = pages[block.Page] || [];
    pages[block.Page].push(block);
  });
  return pages;
};
var ReactOCRRenderer = function ReactOCRRenderer(_ref) {
  var file = _ref.file,
    type = _ref.type,
    _ref$ocrData = _ref.ocrData,
    ocrData = _ref$ocrData === void 0 ? {
      Blocks: [],
      DocumentMetadata: {
        Pages: 0
      }
    } : _ref$ocrData,
    _ref$searchText = _ref.searchText,
    searchText = _ref$searchText === void 0 ? [] : _ref$searchText,
    _ref$highlightedBlock = _ref.highlightedBlockTypes,
    highlightedBlockTypes = _ref$highlightedBlock === void 0 ? [BlockType.WORD] : _ref$highlightedBlock,
    customTextRenderComponent = _ref.customTextRenderComponent,
    customPageRenderComponent = _ref.customPageRenderComponent,
    size = _ref.size,
    showOnlyResultPages = _ref.showOnlyResultPages,
    showOCROverlay = _ref.showOCROverlay,
    emptyComponent = _ref.emptyComponent;
  var _useReactOCRRenderer = useReactOCRRenderer(),
    pageGroups = _useReactOCRRenderer.pages,
    setPages = _useReactOCRRenderer.setPages,
    scrollToBlock = _useReactOCRRenderer.scrollToBlock;
  var _ref2 = ocrData || {
      Blocks: [],
      DocumentMetadata: {
        Pages: 0
      }
    },
    _ref2$Blocks = _ref2.Blocks,
    Blocks = _ref2$Blocks === void 0 ? [] : _ref2$Blocks;
  var cleanedSearchText = useMemo(function () {
    if (typeof searchText === 'string') {
      return [searchText.toLowerCase()];
    }
    if (typeof searchText === 'number') {
      return [searchText.toString()];
    }
    return searchText.map(function (el) {
      return el.toString().toLowerCase();
    });
  }, [searchText]);
  useEffect(function () {
    if (!Blocks.length) {
      return;
    }
    var groupedPages = groupPages(Blocks.filter(function (block) {
      if (!highlightedBlockTypes.includes(block.BlockType)) {
        return false;
      }
      var textCheck;
      if (typeof cleanedSearchText === 'string') {
        textCheck = !block.Text.toLowerCase().includes(cleanedSearchText);
      }
      if (typeof cleanedSearchText === 'number') {
        textCheck = !block.Text.toLowerCase().includes("" + cleanedSearchText);
      }
      if (Array.isArray(cleanedSearchText)) {
        textCheck = !cleanedSearchText.some(function (el) {
          return block.Text.toLowerCase().includes("" + el);
        });
      }
      if (cleanedSearchText.length && textCheck) {
        return false;
      }
      return true;
    }));
    setPages(groupedPages);
  }, [cleanedSearchText, Blocks].concat(highlightedBlockTypes));
  useEffect(function () {
    scrollToBlock(0);
  }, [pageGroups]);
  if (Object.keys(pageGroups).length === 0 && showOnlyResultPages) {
    return emptyComponent !== null && emptyComponent !== void 0 ? emptyComponent : React.createElement("div", null, "No results found");
  }
  if (type === FileType.PDF) {
    return React.createElement(OCRPdfRenderer, {
      file: file,
      ocrData: pageGroups,
      customTextRenderComponent: customTextRenderComponent,
      customPageRenderComponent: customPageRenderComponent,
      size: size,
      showOnlyResultPages: showOnlyResultPages,
      showOCROverlay: showOCROverlay
    });
  }
  return React.createElement("div", null, file);
};

export default ReactOCRRenderer;
export { BlockType, FileType, ReactOCRRendererContextProvider, useReactOCRRenderer };
//# sourceMappingURL=index.modern.js.map
